package cn.wisenergy.service.app.impl;

import cn.wisenergy.common.utils.R;
import cn.wisenergy.mapper.PayRecordMapper;
import cn.wisenergy.model.app.PayRecord;
import cn.wisenergy.model.dto.PayPageDto;
import cn.wisenergy.model.vo.AddLimitVo;
import cn.wisenergy.service.app.TestWxPayService;
import cn.wisenergy.service.app.UserLimitService;
import cn.wisenergy.service.util.MapToXmlUtils;
import cn.wisenergy.service.util.WxPayUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.fasterxml.jackson.core.JsonProcessingException;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.RandomStringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import springfox.documentation.spring.web.json.Json;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;
import java.util.*;


/**
 * @author chengyan
 */
@Service
@Slf4j
public class TestWxPayServiceImpl implements TestWxPayService {
    //统一下单接口链接
    public static String basePath = "https://api.mch.weixin.qq.com/pay/unifiedorder";

    @Autowired
    private UserLimitService userLimitService;
    @Autowired
    private PayRecordMapper payRecordMapper;

    /***
     * pc微信支付总流程
     * @param payPageDto
     * @param response
     * @return
     */
    @Override
    public R<String> wxPay(PayPageDto payPageDto, HttpServletResponse response) {
        if(null==payPageDto || null==payPageDto.getTotal()){
            return R.error("入参不能为空");
        }
        ServletOutputStream sos = null;
        String tradeNo = "21" + System.currentTimeMillis();
        //生成预支付订单，充值结果设为失败（result=1）
        try {
            //生成【统一下单API】所需参数的接口
            String orderInfo = WxPayUtil.createOrderInfo(payPageDto,tradeNo);
            System.out.println(orderInfo);
            //调用统一下单接口
            Map<String, String> map = unifiedOrder(orderInfo);
            String return_msg = map.get("return_msg");//返回信息
            String return_code = map.get("return_code");//状态码
            String result_code = map.get("result_code");//业务结果
            String code_url = map.get("code_url");
            System.out.println("return_msg:"+return_msg);

            log.info("支付返回信息："+return_msg,return_code);
            if (null != map && "SUCCESS".equals(return_code) && "SUCCESS".equals(result_code)) {
                PayRecord payRecord = new PayRecord();
                payRecord.setMoney(payPageDto.getTotal());
                payRecord.setResult(1);
                payRecord.setTradeNo(tradeNo);
                payRecord.setType(payPageDto.getPayType());
                payRecord.setUserId(payPageDto.getUserId());
                payRecordMapper.add(payRecord);
            }
            sos = response.getOutputStream();
            //QRCodeUtil.encode(code_url, sos);//调用生成二维码的方法
            System.out.println("code_url："+code_url);
            return R.ok("0",code_url);
        } catch (Exception e) {
            e.printStackTrace();
            return R.error("获取code_url失败");
        }
    }

    /***
     * h5微信支付总流程
     * @param payPageDto
     * @param request
     * @param response
     * @return
     * @throws JsonProcessingException
     */
    @Override
    public R<String> h5WxPay(PayPageDto payPageDto, HttpServletRequest request, HttpServletResponse response)  {
        if(null==payPageDto || null==payPageDto.getTotal()){
            return R.error("入参不能为空");
        }
        ServletOutputStream sos = null;
        String tradeNo = "21" + System.currentTimeMillis();
        //生成预支付订单，充值结果设为失败（result=1）
        try {
            //生成【统一下单API】所需参数的接口
            String orderInfo = WxPayUtil.createOrderInfoH5(payPageDto,tradeNo,request);
            System.out.println(orderInfo);
            //调用统一下单接口
            Map<String, String> map = unifiedOrder(orderInfo);
            String return_msg = map.get("return_msg");//返回信息
            String return_code = map.get("return_code");//状态码
            String result_code = map.get("result_code");//业务结果
            String urlString = URLEncoder.encode("http://www.baidu.com", "GBK");
            String mweb_url = map.get("mweb_url")+"&redirect_url="+urlString;
            log.info("支付返回信息：",return_msg,"状态码:",return_code);
            if (null != map && "SUCCESS".equals(return_code) && "SUCCESS".equals(result_code)) {
                PayRecord payRecord = new PayRecord();
                payRecord.setMoney(payPageDto.getTotal());
                payRecord.setResult(1);
                payRecord.setTradeNo(tradeNo);
                payRecord.setType(payPageDto.getPayType());
                payRecord.setUserId(payPageDto.getUserId());
                payRecordMapper.add(payRecord);
            }
            System.out.println("mweb_url："+mweb_url);
            return R.ok("0",mweb_url);
        } catch (Exception e) {
            e.printStackTrace();
            return R.error("获取mweb_url失败");
        }
    }

    /***
     * 小程序微信支付总流程
     * @param payPageDto
     * @param request
     * @param response
     * @return
     * @throws JsonProcessingException
     */
    @Override
    public R<String> WxPayApplets(PayPageDto payPageDto, HttpServletRequest request, HttpServletResponse response)  {
        if(null==payPageDto || null==payPageDto.getTotal()){
            return R.error("入参不能为空");
        }
        //随机字符串
        String nonce_str = RandomStringUtils.randomAlphanumeric(16);
        String tradeNo = "21" + System.currentTimeMillis();
        //生成预支付订单，充值结果设为失败（result=1）
        try {
            //生成【统一下单API】所需参数的接口
            String orderInfo = WxPayUtil.createOrderInfoWx(payPageDto,tradeNo,request,nonce_str);
            System.out.println(orderInfo);
            //调用统一下单接口
            Map<String, String> map = unifiedOrder(orderInfo);
            String return_msg = map.get("return_msg");//返回信息
            String return_code = map.get("return_code");//状态码
            String result_code = map.get("result_code");//业务结果

            log.info("支付返回信息：",return_msg,"状态码:",return_code);
            if (null != map && "SUCCESS".equals(return_code) && "SUCCESS".equals(result_code)) {
                PayRecord payRecord = new PayRecord();
                payRecord.setMoney(payPageDto.getTotal());
                payRecord.setResult(1);
                payRecord.setTradeNo(tradeNo);
                payRecord.setType(payPageDto.getPayType());
                payRecord.setUserId(payPageDto.getUserId());
                payRecordMapper.add(payRecord);
                SortedMap<String,String> mapParams = new TreeMap<String,String>();
                mapParams.put("return_code", return_code);
                mapParams.put("return_msg", return_msg);
                String prepay_id = map.get("prepay_id");//返回的预付单信息   小程序端使用
                mapParams.put("nonceStr", nonce_str);
                mapParams.put("package", "prepay_id=" + prepay_id);
                Long timeStamp = new Date().getTime();
                mapParams.put("timeStamp", timeStamp + "");//这边要将返回的时间戳转化成字符串，不然小程序端调用wx.requestPayment方法会报签名错误
                String characterEncoding = "UTF-8";
                String paySign=WxPayUtil.createSign(characterEncoding,mapParams);
                mapParams.put("paySign", paySign);
                mapParams.put("appid", "");


            }
            System.out.println("mweb_url：");
            return R.ok("0");
        } catch (Exception e) {
            e.printStackTrace();
            return R.error("获取mweb_url失败");
        }
    }


    /***
     * 微信统一下单
     * @param info
     * @return
     * @throws Exception
     */
    public Map<String, String> unifiedOrder(String info) throws Exception {
        HttpURLConnection conn = (HttpURLConnection) new URL(basePath).openConnection();
        //加入数据
        conn.setRequestMethod("POST");
        conn.setDoOutput(true);

        BufferedOutputStream buffOutStr = new BufferedOutputStream(conn.getOutputStream());
        buffOutStr.write(info.getBytes("UTF-8"));
        buffOutStr.flush();
        buffOutStr.close();

        //获取输入流
        BufferedReader reader = new BufferedReader(new InputStreamReader(conn.getInputStream(), "UTF-8"));

        String line = null;
        StringBuffer sb = new StringBuffer();
        while ((line = reader.readLine()) != null) {
            sb.append(line);
        }
        Map<String, String> map = MapToXmlUtils.xmlToMap(sb.toString());
       return map;
    }



    /***
     * 微信支付成功回调
     * @param request
     * @param response
     * @return
     */
    @Override
    public R<String> wxPayCallBack(HttpServletRequest request, HttpServletResponse response) {
        System.out.println("回调成功");
        try {
            InputStream inStream = request.getInputStream();
            ByteArrayOutputStream outSteam = new ByteArrayOutputStream();
            byte[] buffer = new byte[1024];
            int len = 0;
            while ((len = inStream.read(buffer)) != -1) {
                outSteam.write(buffer, 0, len);
            }
            outSteam.close();
            inStream.close();
            String result = new String(outSteam.toByteArray(), "utf-8");// 获取微信调用我们notify_url的返回信息
            Map<String, String> map = MapToXmlUtils.xmlToMap(result);
            if (map.get("result_code").equalsIgnoreCase("SUCCESS")) {
                //返回成功后修改订单充值结果为0,保存查询次数
                QueryWrapper<PayRecord> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("trade_no",map.get("out_trade_no"));
                PayRecord payRecord = payRecordMapper.selectOne(queryWrapper);
                AddLimitVo addLimitVo = new AddLimitVo();
                addLimitVo.setTradeNo(map.get("out_trade_no"));
                addLimitVo.setPayMoney(payRecord.getMoney());
                addLimitVo.setPayType(2);
                addLimitVo.setUserId(payRecord.getUserId());
                R<Boolean> booleanR = userLimitService.addLimit(addLimitVo);
                if (booleanR.getCode()==1) {
                    return R.error("用户查询次数添加失败");
                }
                return R.ok(0,"充值成功");
            }
            return R.ok(1,"充值失败");
        } catch (Exception e) {
            return R.ok(1,"充值失败");
        }
    }
}
